/*
 * bits32/stdintlimits.h
 */

#define INT_FAST16_MIN	INT32_MIN
#define INT_FAST32_MIN	INT32_MIN
#define INT_FAST16_MAX	INT32_MAX
#define INT_FAST32_MAX	INT32_MAX
#define UINT_FAST16_MAX UINT32_MAX
#define UINT_FAST32_MAX UINT32_MAX

#define INTPTR_MIN	INT32_MIN
#define INTPTR_MAX	INT32_MAX
#define UINTPTR_MAX	UINT32_MAX

#define PTRDIFF_MIN	INT32_MIN
#define PTRDIFF_MAX	INT32_MAX

/* sig_atomic_t limit */
# define SIG_ATOMIC_MIN         INT32_MIN //(-2147483647-1)
# define SIG_ATOMIC_MAX         INT32_MAX //(2147483647)
/* size_t limit */
# define SIZE_MAX		UINT32_MAX //(4294967295U)
