/*
 * sys/types.h
 */

#ifndef _SYS_TYPES_H
#define _SYS_TYPES_H

#include <klibc/compiler.h>
#include <stddef.h>
#include <stdint.h>

typedef ptrdiff_t ssize_t;
typedef int mode_t;
typedef size_t off_t;

#endif
