/*
 * bits64/limits.h
 */

#ifndef _BITSIZE64_LIMITS_H
#define _BITSIZE64_LIMITS_H

#define LONG_BIT	64

#define LONG_MIN	(-9223372036854775807L-1)
#define LONG_MAX	9223372036854775807L
#define ULONG_MAX	18446744073709551615UL

#endif				/* _BITSIZE_LIMITS_H */
