h3 code {
	font-family: inherit ;
	font-size: inherit ;
}

pre, code {
	font-size: 12pt ;
}

span.apii {
	float: right ;
	font-family: inherit ;
	font-style: normal ;
	font-size: small ;
	color: gray ;
}

p+h1, ul+h1 {
	font-style: normal ;
	padding-top: 0.4em ;
	padding-bottom: 0.4em ;
	padding-left: 16px ;
	margin-left: -16px ;
	background-color: #D0D0FF ;
	border-radius: 8px ;
	border: solid #000080 1px ;
}
