
1 VERSIONINFO
FILEVERSION 0,0,0,1
PRODUCTVERSION 0,0,0,1
FILEOS 0x40004
FILETYPE 0x1
  {
    BLOCK "StringFileInfo"
      {
        BLOCK "040904B0"
          {
            VALUE "CompanyName", "Shao Miller"
            VALUE "FileDescription", "NTFS File Sector Info"
            VALUE "FileVersion", "0.0.0.1 (Aug-12-2011)"
            VALUE "InternalName", "NTFSTest"
            VALUE "LegalCopyright", "© 2011 Shao Miller. All rights reserved."
            VALUE "OriginalFilename", "NTFSTEST.EXE"
            VALUE "ProductName", "Utilities"
            VALUE "ProductVersion", "0.0.0.1"
          }
      }
  }

#if 0
1 ICON "ntfstest.ico"
#endif
