/* ----------------------------------------------------------------------- *
 *
 *   Copyright 2002-2008 H. Peter Anvin - All Rights Reserved
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, Inc., 53 Temple Place Ste 330,
 *   Boston MA 02111-1307, USA; either version 2 of the License, or
 *   (at your option) any later version; incorporated herein by reference.
 *
 * ----------------------------------------------------------------------- */

/*
 * version.h
 *
 * MEMDISK version data
 */

#ifndef MEMDISK_VERSION_H
#define MEMDISK_VERSION_H

#define FIRSTYEAR "2001"
#define COPYYEAR  "2008"

#endif
